/*  This file is part of Tapir.

    Tapir is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Tapir is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tapir; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.elvum.tapir;

class QPSKModulator {
	jfftw.real.Plan BackwardFFT; //frequency to time
	private final int FFTSIZE=256;
	private int m_SamplingFrequency;	//Hz
	private float m_SymbolLength;			//s
	private int m_SymbolBufferSamples;
	private float m_GuardInterval;		//s
	private int m_GuardIntervalSamples;
	private short [] m_GuardIntervalBuffer;
	private short [] m_SymbolBuffer;
	
	protected QPSKModulator(int SamplingFrequency, float SymbolLength, float GuardInterval) {
		BackwardFFT = new jfftw.real.Plan(FFTSIZE, jfftw.Plan.COMPLEX_TO_REAL);
		m_SamplingFrequency = SamplingFrequency;
		m_SymbolLength = SymbolLength;
		m_GuardInterval = GuardInterval;
		m_SymbolBufferSamples = (int)(m_SymbolLength * m_SamplingFrequency);
		m_SymbolBufferSamples = m_SymbolBufferSamples + FFTSIZE - (m_SymbolBufferSamples % FFTSIZE); //overallocates if SymbolBufferSamples is already a multiple of FFTSIZE, I know...
		renderGuardInterval();
	}

	public short [] createSymbol(int b1, int b2) {
		m_SymbolBuffer = new short[m_SymbolBufferSamples];
		double [] FrequencySpaceBuffer = new double[FFTSIZE];
		
		//Set frequency-space components
		if ( b1 == 0 ) FrequencySpaceBuffer[1]=(double)-1;
		else FrequencySpaceBuffer[1]=(double)1;
		if ( b2 == 0 ) FrequencySpaceBuffer[FFTSIZE-1]=(double)-1;
		else FrequencySpaceBuffer[FFTSIZE-1]=(double)1;

/*		double [] zzz = new double[FFTSIZE];
		for (int i=0;i<FFTSIZE;i++) {
			zzz[i] = FrequencySpaceBuffer[i];
		}
*/		
		double [] TimeSpaceBuffer = new double[FFTSIZE];
		TimeSpaceBuffer = BackwardFFT.transform(FrequencySpaceBuffer);
			
/*		for (int i=0;i<FFTSIZE;i++) {
			System.out.println(TimeSpaceBuffer[i]+"	"+zzz[i]);
		}
*/
		//scale output
		for (int i=0;i<FFTSIZE;i++) {
			TimeSpaceBuffer[i]=TimeSpaceBuffer[i]*10000; // ~65536/6 - KLUDGE
		}
		for (int i=0;i<m_SymbolBufferSamples;i++) {
			m_SymbolBuffer[i] = (short)TimeSpaceBuffer[i % FFTSIZE];
		}
		return m_SymbolBuffer;
	}

	public int symbolBufferSamples() {
		return m_SymbolBufferSamples;
	}

	public int guardIntervalSamples() {
		return m_GuardIntervalSamples;
	}
	public short [] createGuardInterval() {
		return m_GuardIntervalBuffer;
	}

	private void renderGuardInterval() {
		m_GuardIntervalSamples = (int)(m_GuardInterval * m_SamplingFrequency);
		m_GuardIntervalBuffer = new short[m_GuardIntervalSamples];
		for (int i=0;i<m_GuardIntervalSamples;i++) {
			m_GuardIntervalBuffer[i] = 0;
		}			
	}

}
