/*  This file is part of Tapir.

    Tapir is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Tapir is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tapir; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package net.elvum.tapir;
import java.io.*;
import java.nio.*;
import java.util.*;

class SymbolStream2bps {
	private Vector m_bit1 = new Vector();
	private Vector m_bit2 = new Vector();
	private short [] m_symbols;
	private QPSKModulator m_mod;
	private boolean m_rendered=false;
	
	protected SymbolStream2bps(QPSKModulator mod, String filename) {
		m_mod=mod;
		m_rendered=false;
		FileInputStream inputFile;
		try {
			inputFile = new FileInputStream(filename);
		} catch (FileNotFoundException e) {
			System.out.println("Error: file not found ("+filename+")");
			return;
		}
		try {
			int nextByte = inputFile.read();
			int i = 0;
			while(nextByte != -1) {
				for (int j=0;j<4;j++) {
					m_bit1.add(new Integer(nextByte & 1));
					nextByte = nextByte >> 1;
					m_bit2.add(new Integer(nextByte & 1));
					nextByte = nextByte >> 1;
				}				
				nextByte = inputFile.read();
			}
		} catch (IOException e) {
			System.out.println("Error: cannot read from file ("+filename+")");
			return;
		}
	}

	public void render() {

		if (m_bit1.size() != m_bit2.size()) {
			System.out.println("Fatal error: symbol bit buffer size mismatch.");
			System.exit(1);
		}
		int symbolInterval = m_mod.symbolBufferSamples() + m_mod.guardIntervalSamples();
		m_symbols = new short [ symbolInterval * m_bit1.size() ];
		Iterator ItBit1 = m_bit1.iterator();
		Iterator ItBit2 = m_bit2.iterator();
		int symbolNumber = 0;
		int j = 0;
		while(ItBit1.hasNext() && ItBit2.hasNext()) {
			short [] symbol = m_mod.createSymbol(((Integer)ItBit1.next()).intValue(), ((Integer)ItBit2.next()).intValue());
			for (int i=0;i<symbol.length;i++) {
				m_symbols[j++]=symbol[i];
			}
			short [] guardInterval = m_mod.createGuardInterval();
			for (int i=0;i<guardInterval.length;i++) {
				m_symbols[j++]=guardInterval[i];
			}
		}			
		m_rendered=true;
	}

	public short [] getStream() {
		if (!m_rendered) render();
		return m_symbols;
	}
}
